# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: agentuniverse_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x61gentuniverse_service.proto\x12\ragentuniverse\"H\n\x13\x41gentServiceRequest\x12\x12\n\nservice_id\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x01(\t\x12\r\n\x05saved\x18\x03 \x01(\x08\"\\\n\x14\x41gentServiceResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0f\n\x07success\x18\x02 \x01(\x08\x12\x12\n\nrequest_id\x18\x03 \x01(\t\x12\x0e\n\x06result\x18\x04 \x01(\t\"(\n\x12\x41gentResultRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t2\xaa\x02\n\x14\x41gentUniverseService\x12V\n\x0bservice_run\x12\".agentuniverse.AgentServiceRequest\x1a#.agentuniverse.AgentServiceResponse\x12\\\n\x11service_run_async\x12\".agentuniverse.AgentServiceRequest\x1a#.agentuniverse.AgentServiceResponse\x12\\\n\x12service_run_result\x12!.agentuniverse.AgentResultRequest\x1a#.agentuniverse.AgentServiceResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'agentuniverse_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_AGENTSERVICEREQUEST']._serialized_start=46
  _globals['_AGENTSERVICEREQUEST']._serialized_end=118
  _globals['_AGENTSERVICERESPONSE']._serialized_start=120
  _globals['_AGENTSERVICERESPONSE']._serialized_end=212
  _globals['_AGENTRESULTREQUEST']._serialized_start=214
  _globals['_AGENTRESULTREQUEST']._serialized_end=254
  _globals['_AGENTUNIVERSESERVICE']._serialized_start=257
  _globals['_AGENTUNIVERSESERVICE']._serialized_end=555
# @@protoc_insertion_point(module_scope)
